searchData={"items":[{"type":"module","title":"Protoss","doc":"Protoss is a utility library that allows you to define protocols implementations\nwhich automatically delegate back to the module that is the caller of the protocol.\n\n#","ref":"Protoss.html"},{"type":"module","title":"Automatic delegation of protocol functions - Protoss","doc":"The primary purpose of `Protoss` is to automatically delegate protocol functions.\nAs an example, here is standard elixir code for implementing a protocol:\n\n```elixir\ndefprotocol Proto do\n  def unwrap(term)\nend\n\ndefmodule MyStruct do\n  defstruct [:value]\n  defimpl Proto do\n    def unwrap(term), do: term.value\n  end\nend\n```\n\nProtoss makes this easier to understand by hiding the `defimpl` boilerplate module.\nThis is a natural fit for Elixir's convention of having module functions whose\nfirst parameter is the module's datatype.\n\n```elixir\nuse Protoss\ndefprotocol Proto do\n  def unwrap(term)\nend\n\ndefmodule MyStruct do\n  use Proto\n  defstruct [:value]\n  def unwrap(term), do: term.value\nend\n```\n\n#","ref":"Protoss.html#module-automatic-delegation-of-protocol-functions"},{"type":"module","title":"Directly delegated functions. - Protoss","doc":"Protoss also allows you to define functions that get directly delegated to the\nimplementation module, that are then logically associated with the protocol, and\nwhose existence is checked by the compiler using the protocol behaviour.\n\nA common use case would be a `from_json` function which reifies json data into a\nstruct.\n\n##","ref":"Protoss.html#module-directly-delegated-functions"},{"type":"module","title":"Example: - Protoss","doc":"```elixir\nuse Protoss\ndefprotocol Proto do\n  def unwrap(term)\n  defdelegate from_json(module, json)\nend\n\ndefmodule MyStruct do\n  use Proto\n  defstruct [:value]\n  def unwrap(term), do: term.value\n\n  # NOTE the arity of the function\n  def from_json(%{\"value\" => v}), do: %__MODULE__{value: v}\nend\n```\n\n#","ref":"Protoss.html#module-example"},{"type":"module","title":"Protocol body functions - Protoss","doc":"Finally, protoss allows you to write arbitrary functions in the protocol body\nthat aren't necessarily part of the protocol itself, using the `after` keyword.  \nThis could be useful to reduce code duplication if you have a common processing \nstep that must occur in conjunction with a pseudo-private protocol function, \nor even if you simply have a thematically relevant function that you would like \nto incorporate  into the same namespace.\n\n```elixir\nuse Protoss\ndefprotocol Proto do\n  def _unwrap_impl(term)\nafter\n  def unwrap(struct) do\n    struct\n    |> _unwrap_impl()\n    |> SomeOtherModule.process()\n  end\nend\n\ndefmodule MyStruct do\n  use Proto\n  defstruct [:value]\n  def _unwrap_impl(term), do: term.value\nend\n```","ref":"Protoss.html#module-protocol-body-functions"},{"type":"macro","title":"Protoss.defdelegate/1","doc":"","ref":"Protoss.html#defdelegate/1"},{"type":"macro","title":"Protoss.defprotocol/2","doc":"","ref":"Protoss.html#defprotocol/2"},{"type":"extras","title":"Protoss","doc":"# Protoss\n\n![tests](https://github.com/ityonemo/protoss/actions/workflows/basic-workflow.yml/badge.svg)\n\nEvil and Powerful Protocols for Elixir\n\nProtocols are just modules!  And sometimes you just want to do\nmodule things in protocols.  And when you're in control of both\nthe protocol and the module you're implementing for, colocating\nthe code in the module you're writing *just makes sense*.  This\nlibrary does this automatically for you.","ref":"readme.html"},{"type":"extras","title":"Usage - Protoss","doc":"#","ref":"readme.html#usage"},{"type":"extras","title":"Protocol definition - Protoss","doc":"```elixir\nuse Protoss\n\ndefprotocol MyProtocol do\n  # this works like a normal protocol forward definition.\n  def fun_must_be_implemented(argument)\n\n  # delegations forward to the passed module.  Currently\n  # this is not currently runtime-checked, but it will be \n  # in the future under a flag.\n  #\n  # call as:\n  # `MyProtocol.from_json(MyStruct, ...)`\n  #\n  defdelegate from_json(module, argument)\n\n  # after block separates protocol code from module code\nafter\n\n  # struct protocol implementations MUST implement these\n  # extra callbacks:\n  @callback extra_callback() :: term\n\n  # this can be called directly as: `MyProtocol.root_function()`\n  def root_function do\n    :ok\n  end\nend\n```\n\n#","ref":"readme.html#protocol-definition"},{"type":"extras","title":"Protocol implementation - Protoss","doc":"In your (struct) protocols, the implementation of `MyProtocol` from above\nwould look like this.\n\n```elixir\ndefmodule MyStruct do\n  defstruct [:value]\n\n  use MyProtocol\n\n  @impl MyProtocol\n  def fun_must_be_implemented(%__MODULE__{value: value}), do: value\n\n  # note the arity change!\n  @impl MyProtocol\n  def from_json(%{\"value\" => value}), do: %__MODULE__{value: value}\n\n  @impl MyProtocol\n  def extra_callback, do: \"I was required to implement this\"\nend\n```\n\n#","ref":"readme.html#protocol-implementation"},{"type":"extras","title":"Formatter changes - Protoss","doc":"In order to get proper formatting, you should add the following option to your `.formatter.exs`\n\n`locals_without_parens: [defdelegate: 1]`","ref":"readme.html#formatter-changes"},{"type":"extras","title":"Installation - Protoss","doc":"The package is available on hex and can be installed by adding `protoss` to your \nlist of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:protoss, \"~> 0.2\"}\n  ]\nend\n```\n\nDocumentation can be found at  .","ref":"readme.html#installation"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,49,46,50]}}